@@ -1,18 +1,21 @@
-/* $Id: memblock.c 1328 2006-08-23 07:58:07Z ossman $ */
+/* $Id: memblock.c 1459 2007-05-27 20:38:14Z lennart $ */
 
 /***
   This file is part of PulseAudio.
- 
+
+  Copyright 2004-2006 Lennart Poettering
+  Copyright 2006 Pierre Ossman <ossman@cendio.se> for Cendio AB
+
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of the
   License, or (at your option) any later version.
- 
+
   PulseAudio is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.
- 
+
   You should have received a copy of the GNU Lesser General Public
   License along with PulseAudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -71,7 +74,7 @@ struct memexport_slot {
 
 struct pa_memexport {
     pa_mempool *pool;
-    
+
     struct memexport_slot slots[PA_MEMEXPORT_SLOTS_MAX];
     PA_LLIST_HEAD(struct memexport_slot, free_slots);
     PA_LLIST_HEAD(struct memexport_slot, used_slots);
@@ -101,8 +104,7 @@ struct pa_mempool {
 
     /* A list of free slots that may be reused */
     PA_LLIST_HEAD(struct mempool_slot, free_slots);
-    PA_LLIST_HEAD(struct mempool_slot, used_slots);
-    
+
     pa_mempool_stat stat;
 };
 
@@ -112,49 +114,50 @@ static void stat_add(pa_memblock*b) {
     assert(b);
     assert(b->pool);
 
-    b->pool->stat.n_allocated ++;
-    b->pool->stat.n_accumulated ++;
-    b->pool->stat.allocated_size += b->length;
-    b->pool->stat.accumulated_size += b->length;
+    pa_atomic_inc(&b->pool->stat.n_allocated);
+    pa_atomic_add(&b->pool->stat.allocated_size, b->length);
+
+    pa_atomic_inc(&b->pool->stat.n_accumulated);
+    pa_atomic_add(&b->pool->stat.accumulated_size, b->length);
 
     if (b->type == PA_MEMBLOCK_IMPORTED) {
-        b->pool->stat.n_imported++;
-        b->pool->stat.imported_size += b->length;
+        pa_atomic_inc(&b->pool->stat.n_imported);
+        pa_atomic_add(&b->pool->stat.imported_size, b->length);
     }
 
-    b->pool->stat.n_allocated_by_type[b->type]++;
-    b->pool->stat.n_accumulated_by_type[b->type]++;
+    pa_atomic_inc(&b->pool->stat.n_allocated_by_type[b->type]);
+    pa_atomic_inc(&b->pool->stat.n_accumulated_by_type[b->type]);
 }
 
 static void stat_remove(pa_memblock *b) {
     assert(b);
     assert(b->pool);
 
-    assert(b->pool->stat.n_allocated > 0);
-    assert(b->pool->stat.allocated_size >= b->length);
-           
-    b->pool->stat.n_allocated --;
-    b->pool->stat.allocated_size -= b->length;
+    assert(pa_atomic_load(&b->pool->stat.n_allocated) > 0);
+    assert(pa_atomic_load(&b->pool->stat.allocated_size) >= (int) b->length);
+
+    pa_atomic_dec(&b->pool->stat.n_allocated);
+    pa_atomic_sub(&b->pool->stat.allocated_size,  b->length);
 
     if (b->type == PA_MEMBLOCK_IMPORTED) {
-        assert(b->pool->stat.n_imported > 0);
-        assert(b->pool->stat.imported_size >= b->length);
-        
-        b->pool->stat.n_imported --;
-        b->pool->stat.imported_size -= b->length;
+        assert(pa_atomic_load(&b->pool->stat.n_imported) > 0);
+        assert(pa_atomic_load(&b->pool->stat.imported_size) >= (int) b->length);
+
+        pa_atomic_dec(&b->pool->stat.n_imported);
+        pa_atomic_sub(&b->pool->stat.imported_size, b->length);
     }
 
-    b->pool->stat.n_allocated_by_type[b->type]--;
+    pa_atomic_dec(&b->pool->stat.n_allocated_by_type[b->type]);
 }
 
 static pa_memblock *memblock_new_appended(pa_mempool *p, size_t length);
 
 pa_memblock *pa_memblock_new(pa_mempool *p, size_t length) {
     pa_memblock *b;
-    
+
     assert(p);
     assert(length > 0);
-    
+
     if (!(b = pa_memblock_new_pool(p, length)))
         b = memblock_new_appended(p, length);
 
@@ -170,7 +173,7 @@ static pa_memblock *memblock_new_appended(pa_mempool *p, size_t length) {
     b = pa_xmalloc(sizeof(pa_memblock) + length);
     b->type = PA_MEMBLOCK_APPENDED;
     b->read_only = 0;
-    b->ref = 1;
+    PA_REFCNT_INIT(b);
     b->length = length;
     b->data = (uint8_t*) b + sizeof(pa_memblock);
     b->pool = p;
@@ -190,11 +193,10 @@ static struct mempool_slot* mempool_allocate_slot(pa_mempool *p) {
         slot = (struct mempool_slot*) ((uint8_t*) p->memory.ptr + (p->block_size * p->n_init++));
     else {
         pa_log_debug("Pool full");
-        p->stat.n_pool_full++;
+        pa_atomic_inc(&p->stat.n_pool_full);
         return NULL;
     }
 
-    PA_LLIST_PREPEND(struct mempool_slot, p->used_slots, slot);
     return slot;
 }
 
@@ -232,28 +234,28 @@ pa_memblock *pa_memblock_new_pool(pa_mempool *p, size_t length) {
 
         if (!(slot = mempool_allocate_slot(p)))
             return NULL;
-        
+
         b = mempool_slot_data(slot);
         b->type = PA_MEMBLOCK_POOL;
         b->data = (uint8_t*) b + sizeof(pa_memblock);
-        
+
     } else if (p->block_size - sizeof(struct mempool_slot) >= length) {
 
         if (!(slot = mempool_allocate_slot(p)))
             return NULL;
-        
+
         b = pa_xnew(pa_memblock, 1);
         b->type = PA_MEMBLOCK_POOL_EXTERNAL;
         b->data = mempool_slot_data(slot);
     } else {
         pa_log_debug("Memory block too large for pool: %u > %u", length, p->block_size - sizeof(struct mempool_slot));
-        p->stat.n_too_large_for_pool++;
+        pa_atomic_inc(&p->stat.n_too_large_for_pool);
         return NULL;
     }
 
     b->length = length;
     b->read_only = 0;
-    b->ref = 1;
+    PA_REFCNT_INIT(b);
     b->pool = p;
 
     stat_add(b);
@@ -270,7 +272,7 @@ pa_memblock *pa_memblock_new_fixed(pa_mempool *p, void *d, size_t length, int re
     b = pa_xnew(pa_memblock, 1);
     b->type = PA_MEMBLOCK_FIXED;
     b->read_only = read_only;
-    b->ref = 1;
+    PA_REFCNT_INIT(b);
     b->length = length;
     b->data = d;
     b->pool = p;
@@ -286,11 +288,11 @@ pa_memblock *pa_memblock_new_user(pa_mempool *p, void *d, size_t length, void (*
     assert(d);
     assert(length > 0);
     assert(free_cb);
-    
+
     b = pa_xnew(pa_memblock, 1);
     b->type = PA_MEMBLOCK_USER;
     b->read_only = read_only;
-    b->ref = 1;
+    PA_REFCNT_INIT(b);
     b->length = length;
     b->data = d;
     b->per_type.user.free_cb = free_cb;
@@ -302,19 +304,19 @@ pa_memblock *pa_memblock_new_user(pa_mempool *p, void *d, size_t length, void (*
 
 pa_memblock* pa_memblock_ref(pa_memblock*b) {
     assert(b);
-    assert(b->ref >= 1);
-    
-    b->ref++;
+    assert(PA_REFCNT_VALUE(b) > 0);
+
+    PA_REFCNT_INC(b);
     return b;
 }
 
 void pa_memblock_unref(pa_memblock*b) {
     assert(b);
-    assert(b->ref >= 1);
+    assert(PA_REFCNT_VALUE(b) > 0);
 
-    if ((--(b->ref)) > 0)
+    if (PA_REFCNT_DEC(b) > 0)
         return;
-    
+
     stat_remove(b);
 
     switch (b->type) {
@@ -335,13 +337,13 @@ void pa_memblock_unref(pa_memblock*b) {
             segment = b->per_type.imported.segment;
             assert(segment);
             assert(segment->import);
-            
+
             pa_hashmap_remove(segment->import->blocks, PA_UINT32_TO_PTR(b->per_type.imported.id));
             segment->import->release_cb(segment->import, b->per_type.imported.id, segment->import->userdata);
 
             if (-- segment->n_blocks <= 0)
                 segment_detach(segment);
-            
+
             pa_xfree(b);
             break;
         }
@@ -352,10 +354,9 @@ void pa_memblock_unref(pa_memblock*b) {
 
             slot = mempool_slot_by_ptr(b->pool, b->data);
             assert(slot);
-            
-            PA_LLIST_REMOVE(struct mempool_slot, b->pool->used_slots, slot);
+
             PA_LLIST_PREPEND(struct mempool_slot, b->pool->free_slots, slot);
-            
+
             if (b->type == PA_MEMBLOCK_POOL_EXTERNAL)
                 pa_xfree(b);
 
@@ -371,7 +372,7 @@ void pa_memblock_unref(pa_memblock*b) {
 static void memblock_make_local(pa_memblock *b) {
     assert(b);
 
-    b->pool->stat.n_allocated_by_type[b->type]--;
+    pa_atomic_dec(&b->pool->stat.n_allocated_by_type[b->type]);
 
     if (b->length <= b->pool->block_size - sizeof(struct mempool_slot)) {
         struct mempool_slot *slot;
@@ -379,7 +380,7 @@ static void memblock_make_local(pa_memblock *b) {
         if ((slot = mempool_allocate_slot(b->pool))) {
             void *new_data;
             /* We can move it into a local pool, perfect! */
-            
+
             b->type = PA_MEMBLOCK_POOL_EXTERNAL;
             b->read_only = 0;
 
@@ -397,16 +398,16 @@ static void memblock_make_local(pa_memblock *b) {
     b->data = pa_xmemdup(b->data, b->length);
 
 finish:
-    b->pool->stat.n_allocated_by_type[b->type]++;
-    b->pool->stat.n_accumulated_by_type[b->type]++;
+    pa_atomic_inc(&b->pool->stat.n_allocated_by_type[b->type]);
+    pa_atomic_inc(&b->pool->stat.n_accumulated_by_type[b->type]);
 }
 
 void pa_memblock_unref_fixed(pa_memblock *b) {
     assert(b);
-    assert(b->ref >= 1);
+    assert(PA_REFCNT_VALUE(b) > 0);
     assert(b->type == PA_MEMBLOCK_FIXED);
 
-    if (b->ref > 1)
+    if (PA_REFCNT_VALUE(b) > 1)
         memblock_make_local(b);
 
     pa_memblock_unref(b);
@@ -414,14 +415,14 @@ void pa_memblock_unref_fixed(pa_memblock *b) {
 
 static void memblock_replace_import(pa_memblock *b) {
     pa_memimport_segment *seg;
-    
+
     assert(b);
     assert(b->type == PA_MEMBLOCK_IMPORTED);
 
-    assert(b->pool->stat.n_imported > 0);
-    assert(b->pool->stat.imported_size >= b->length);
-    b->pool->stat.n_imported --;
-    b->pool->stat.imported_size -= b->length;
+    assert(pa_atomic_load(&b->pool->stat.n_imported) > 0);
+    assert(pa_atomic_load(&b->pool->stat.imported_size) >= (int) b->length);
+    pa_atomic_dec(&b->pool->stat.n_imported);
+    pa_atomic_sub(&b->pool->stat.imported_size, b->length);
 
     seg = b->per_type.imported.segment;
     assert(seg);
@@ -455,7 +456,7 @@ pa_mempool* pa_mempool_new(int shared) {
 
     if (p->block_size < ps)
         p->block_size = ps;
-    
+
     p->n_blocks = PA_MEMPOOL_SLOTS_MAX;
 
     assert(p->block_size > sizeof(struct mempool_slot));
@@ -466,11 +467,10 @@ pa_mempool* pa_mempool_new(int shared) {
     }
 
     p->n_init = 0;
-    
+
     PA_LLIST_HEAD_INIT(pa_memimport, p->imports);
     PA_LLIST_HEAD_INIT(pa_memexport, p->exports);
     PA_LLIST_HEAD_INIT(struct mempool_slot, p->free_slots);
-    PA_LLIST_HEAD_INIT(struct mempool_slot, p->used_slots);
 
     memset(&p->stat, 0, sizeof(p->stat));
 
@@ -486,9 +486,9 @@ void pa_mempool_free(pa_mempool *p) {
     while (p->exports)
         pa_memexport_free(p->exports);
 
-    if (p->stat.n_allocated > 0)
+    if (pa_atomic_load(&p->stat.n_allocated) > 0)
         pa_log_warn("WARNING! Memory pool destroyed but not all memory blocks freed!");
-    
+
     pa_shm_free(&p->memory);
     pa_xfree(p);
 }
@@ -501,12 +501,11 @@ const pa_mempool_stat* pa_mempool_get_stat(pa_mempool *p) {
 
 void pa_mempool_vacuum(pa_mempool *p) {
     struct mempool_slot *slot;
-    
+
     assert(p);
 
-    for (slot = p->free_slots; slot; slot = slot->next) {
+    for (slot = p->free_slots; slot; slot = slot->next)
         pa_shm_punch(&p->memory, (uint8_t*) slot + sizeof(struct mempool_slot) - (uint8_t*) p->memory.ptr, p->block_size - sizeof(struct mempool_slot));
-    }
 }
 
 int pa_mempool_get_shm_id(pa_mempool *p, uint32_t *id) {
@@ -516,7 +515,7 @@ int pa_mempool_get_shm_id(pa_mempool *p, uint32_t *id) {
         return -1;
 
     *id = p->memory.id;
-    
+
     return 0;
 }
 
@@ -532,14 +531,14 @@ pa_memimport* pa_memimport_new(pa_mempool *p, pa_memimport_release_cb_t cb, void
 
     assert(p);
     assert(cb);
-    
+
     i = pa_xnew(pa_memimport, 1);
     i->pool = p;
     i->segments = pa_hashmap_new(NULL, NULL);
     i->blocks = pa_hashmap_new(NULL, NULL);
     i->release_cb = cb;
     i->userdata = userdata;
-    
+
     PA_LLIST_PREPEND(pa_memimport, p->imports, i);
     return i;
 }
@@ -553,7 +552,7 @@ static pa_memimport_segment* segment_attach(pa_memimport *i, uint32_t shm_id) {
         return NULL;
 
     seg = pa_xnew(pa_memimport_segment, 1);
-    
+
     if (pa_shm_attach_ro(&seg->memory, shm_id) < 0) {
         pa_xfree(seg);
         return NULL;
@@ -561,7 +560,7 @@ static pa_memimport_segment* segment_attach(pa_memimport *i, uint32_t shm_id) {
 
     seg->import = i;
     seg->n_blocks = 0;
-    
+
     pa_hashmap_put(i->segments, PA_UINT32_TO_PTR(shm_id), seg);
     return seg;
 }
@@ -577,7 +576,7 @@ static void segment_detach(pa_memimport_segment *seg) {
 void pa_memimport_free(pa_memimport *i) {
     pa_memexport *e;
     pa_memblock *b;
-    
+
     assert(i);
 
     /* If we've exported this block further we need to revoke that export */
@@ -591,7 +590,7 @@ void pa_memimport_free(pa_memimport *i) {
 
     pa_hashmap_free(i->blocks, NULL, NULL);
     pa_hashmap_free(i->segments, NULL, NULL);
-    
+
     PA_LLIST_REMOVE(pa_memimport, i->pool->imports, i);
     pa_xfree(i);
 }
@@ -599,23 +598,23 @@ void pa_memimport_free(pa_memimport *i) {
 pa_memblock* pa_memimport_get(pa_memimport *i, uint32_t block_id, uint32_t shm_id, size_t offset, size_t size) {
     pa_memblock *b;
     pa_memimport_segment *seg;
-    
+
     assert(i);
 
     if (pa_hashmap_size(i->blocks) >= PA_MEMIMPORT_SLOTS_MAX)
         return NULL;
 
-    if (!(seg = pa_hashmap_get(i->segments, PA_UINT32_TO_PTR(shm_id)))) 
+    if (!(seg = pa_hashmap_get(i->segments, PA_UINT32_TO_PTR(shm_id))))
         if (!(seg = segment_attach(i, shm_id)))
             return NULL;
 
     if (offset+size > seg->memory.size)
         return NULL;
-    
+
     b = pa_xnew(pa_memblock, 1);
     b->type = PA_MEMBLOCK_IMPORTED;
     b->read_only = 1;
-    b->ref = 1;
+    PA_REFCNT_INIT(b);
     b->length = size;
     b->data = (uint8_t*) seg->memory.ptr + offset;
     b->pool = i->pool;
@@ -625,9 +624,9 @@ pa_memblock* pa_memimport_get(pa_memimport *i, uint32_t block_id, uint32_t shm_i
     pa_hashmap_put(i->blocks, PA_UINT32_TO_PTR(block_id), b);
 
     seg->n_blocks++;
-    
+
     stat_add(b);
-    
+
     return b;
 }
 
@@ -637,7 +636,7 @@ int pa_memimport_process_revoke(pa_memimport *i, uint32_t id) {
 
     if (!(b = pa_hashmap_get(i->blocks, PA_UINT32_TO_PTR(id))))
         return -1;
-    
+
     memblock_replace_import(b);
     return 0;
 }
@@ -645,13 +644,13 @@ int pa_memimport_process_revoke(pa_memimport *i, uint32_t id) {
 /* For sending blocks to other nodes */
 pa_memexport* pa_memexport_new(pa_mempool *p, pa_memexport_revoke_cb_t cb, void *userdata) {
     pa_memexport *e;
-    
+
     assert(p);
     assert(cb);
 
     if (!p->memory.shared)
         return NULL;
-    
+
     e = pa_xnew(pa_memexport, 1);
     e->pool = p;
     PA_LLIST_HEAD_INIT(struct memexport_slot, e->free_slots);
@@ -659,7 +658,7 @@ pa_memexport* pa_memexport_new(pa_mempool *p, pa_memexport_revoke_cb_t cb, void
     e->n_init = 0;
     e->revoke_cb = cb;
     e->userdata = userdata;
-    
+
     PA_LLIST_PREPEND(pa_memexport, p->exports, e);
     return e;
 }
@@ -685,12 +684,12 @@ int pa_memexport_process_release(pa_memexport *e, uint32_t id) {
 
 /*     pa_log("Processing release for %u", id); */
 
-    assert(e->pool->stat.n_exported > 0);
-    assert(e->pool->stat.exported_size >= e->slots[id].block->length);
-    
-    e->pool->stat.n_exported --;
-    e->pool->stat.exported_size -= e->slots[id].block->length;
-    
+    assert(pa_atomic_load(&e->pool->stat.n_exported) > 0);
+    assert(pa_atomic_load(&e->pool->stat.exported_size) >= (int) e->slots[id].block->length);
+
+    pa_atomic_dec(&e->pool->stat.n_exported);
+    pa_atomic_sub(&e->pool->stat.exported_size, e->slots[id].block->length);
+
     pa_memblock_unref(e->slots[id].block);
     e->slots[id].block = NULL;
 
@@ -708,7 +707,7 @@ static void memexport_revoke_blocks(pa_memexport *e, pa_memimport *i) {
     for (slot = e->used_slots; slot; slot = next) {
         uint32_t idx;
         next = slot->next;
-        
+
         if (slot->block->type != PA_MEMBLOCK_IMPORTED ||
             slot->block->per_type.imported.segment->import != i)
             continue;
@@ -724,7 +723,7 @@ static pa_memblock *memblock_shared_copy(pa_mempool *p, pa_memblock *b) {
 
     assert(p);
     assert(b);
-    
+
     if (b->type == PA_MEMBLOCK_IMPORTED ||
         b->type == PA_MEMBLOCK_POOL ||
         b->type == PA_MEMBLOCK_POOL_EXTERNAL) {
@@ -742,7 +741,7 @@ static pa_memblock *memblock_shared_copy(pa_mempool *p, pa_memblock *b) {
 int pa_memexport_put(pa_memexport *e, pa_memblock *b, uint32_t *block_id, uint32_t *shm_id, size_t *offset, size_t * size) {
     pa_shm *memory;
     struct memexport_slot *slot;
-    
+
     assert(e);
     assert(b);
     assert(block_id);
@@ -778,16 +777,16 @@ int pa_memexport_put(pa_memexport *e, pa_memblock *b, uint32_t *block_id, uint32
         assert(b->pool);
         memory = &b->pool->memory;
     }
-        
+
     assert(b->data >= memory->ptr);
     assert((uint8_t*) b->data + b->length <= (uint8_t*) memory->ptr + memory->size);
-    
+
     *shm_id = memory->id;
     *offset = (uint8_t*) b->data - (uint8_t*) memory->ptr;
     *size = b->length;
 
-    e->pool->stat.n_exported ++;
-    e->pool->stat.exported_size += b->length;
+    pa_atomic_inc(&e->pool->stat.n_exported);
+    pa_atomic_add(&e->pool->stat.exported_size, b->length);
 
     return 0;
 }
