@@ -1,18 +1,21 @@
-/* $Id: pstream.c 1300 2006-08-19 18:57:33Z lennart $ */
+/* $Id: pstream.c 1447 2007-05-23 16:30:57Z lennart $ */
 
 /***
   This file is part of PulseAudio.
- 
+
+  Copyright 2004-2006 Lennart Poettering
+  Copyright 2006 Pierre Ossman <ossman@cendio.se> for Cendio AB
+
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of the
   License, or (at your option) any later version.
- 
+
   PulseAudio is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.
- 
+
   You should have received a copy of the GNU Lesser General Public
   License along with PulseAudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -46,6 +49,8 @@
 #include <pulsecore/log.h>
 #include <pulsecore/core-scache.h>
 #include <pulsecore/creds.h>
+#include <pulsecore/mutex.h>
+#include <pulsecore/refcnt.h>
 
 #include "pstream.h"
 
@@ -108,12 +113,13 @@ struct item_info {
 };
 
 struct pa_pstream {
-    int ref;
-    
+    PA_REFCNT_DECLARE;
+
     pa_mainloop_api *mainloop;
     pa_defer_event *defer_event;
     pa_iochannel *io;
     pa_queue *send_queue;
+    pa_mutex *mutex;
 
     int dead;
 
@@ -163,11 +169,14 @@ static int do_read(pa_pstream *p);
 
 static void do_something(pa_pstream *p) {
     assert(p);
-
-    p->mainloop->defer_enable(p->defer_event, 0);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
     pa_pstream_ref(p);
 
+    pa_mutex_lock(p->mutex);
+
+    p->mainloop->defer_enable(p->defer_event, 0);
+
     if (!p->dead && pa_iochannel_is_readable(p->io)) {
         if (do_read(p) < 0)
             goto fail;
@@ -179,25 +188,29 @@ static void do_something(pa_pstream *p) {
             goto fail;
     }
 
+    pa_mutex_unlock(p->mutex);
+
     pa_pstream_unref(p);
     return;
 
 fail:
 
     p->dead = 1;
-    
+
     if (p->die_callback)
         p->die_callback(p, p->die_callback_userdata);
-    
+
+    pa_mutex_unlock(p->mutex);
+
     pa_pstream_unref(p);
 }
 
 static void io_callback(pa_iochannel*io, void *userdata) {
     pa_pstream *p = userdata;
-    
+
     assert(p);
     assert(p->io == io);
-    
+
     do_something(p);
 }
 
@@ -207,7 +220,7 @@ static void defer_callback(pa_mainloop_api *m, pa_defer_event *e, void*userdata)
     assert(p);
     assert(p->defer_event == e);
     assert(p->mainloop == m);
-    
+
     do_something(p);
 }
 
@@ -215,21 +228,23 @@ static void memimport_release_cb(pa_memimport *i, uint32_t block_id, void *userd
 
 pa_pstream *pa_pstream_new(pa_mainloop_api *m, pa_iochannel *io, pa_mempool *pool) {
     pa_pstream *p;
-    
+
     assert(m);
     assert(io);
     assert(pool);
 
     p = pa_xnew(pa_pstream, 1);
-    p->ref = 1;
+    PA_REFCNT_INIT(p);
     p->io = io;
     pa_iochannel_set_callback(io, io_callback, p);
     p->dead = 0;
 
+    p->mutex = pa_mutex_new(1);
+
     p->mainloop = m;
     p->defer_event = m->defer_new(m, defer_callback, p);
     m->defer_enable(p->defer_event, 0);
-    
+
     p->send_queue = pa_queue_new();
     assert(p->send_queue);
 
@@ -256,7 +271,7 @@ pa_pstream *pa_pstream_new(pa_mainloop_api *m, pa_iochannel *io, pa_mempool *poo
     /* We do importing unconditionally */
     p->import = pa_memimport_new(p->mempool, memimport_release_cb, p);
 
-    pa_iochannel_socket_set_rcvbuf(io, 1024*8); 
+    pa_iochannel_socket_set_rcvbuf(io, 1024*8);
     pa_iochannel_socket_set_sndbuf(io, 1024*8);
 
 #ifdef HAVE_CREDS
@@ -285,7 +300,7 @@ static void pstream_free(pa_pstream *p) {
     assert(p);
 
     pa_pstream_close(p);
-    
+
     pa_queue_free(p->send_queue, item_free, NULL);
 
     if (p->write.current)
@@ -293,10 +308,13 @@ static void pstream_free(pa_pstream *p) {
 
     if (p->read.memblock)
         pa_memblock_unref(p->read.memblock);
-    
+
     if (p->read.packet)
         pa_packet_unref(p->read.packet);
 
+    if (p->mutex)
+        pa_mutex_free(p->mutex);
+
     pa_xfree(p);
 }
 
@@ -304,16 +322,18 @@ void pa_pstream_send_packet(pa_pstream*p, pa_packet *packet, const pa_creds *cre
     struct item_info *i;
 
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
     assert(packet);
 
+    pa_mutex_lock(p->mutex);
+
     if (p->dead)
-        return;
-    
+        goto finish;
+
     i = pa_xnew(struct item_info, 1);
     i->type = PA_PSTREAM_ITEM_PACKET;
     i->packet = pa_packet_ref(packet);
-    
+
 #ifdef HAVE_CREDS
     if ((i->with_creds = !!creds))
         i->creds = *creds;
@@ -321,18 +341,24 @@ void pa_pstream_send_packet(pa_pstream*p, pa_packet *packet, const pa_creds *cre
 
     pa_queue_push(p->send_queue, i);
     p->mainloop->defer_enable(p->defer_event, 1);
+
+finish:
+
+    pa_mutex_unlock(p->mutex);
 }
 
 void pa_pstream_send_memblock(pa_pstream*p, uint32_t channel, int64_t offset, pa_seek_mode_t seek_mode, const pa_memchunk *chunk) {
     size_t length, idx;
-    
+
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
     assert(channel != (uint32_t) -1);
     assert(chunk);
 
+    pa_mutex_lock(p->mutex);
+
     if (p->dead)
-        return;
+        goto finish;
 
     length = chunk->length;
     idx = 0;
@@ -340,7 +366,7 @@ void pa_pstream_send_memblock(pa_pstream*p, uint32_t channel, int64_t offset, pa
     while (length > 0) {
         struct item_info *i;
         size_t n;
-        
+
         i = pa_xnew(struct item_info, 1);
         i->type = PA_PSTREAM_ITEM_MEMBLOCK;
 
@@ -348,21 +374,25 @@ void pa_pstream_send_memblock(pa_pstream*p, uint32_t channel, int64_t offset, pa
         i->chunk.index = chunk->index + idx;
         i->chunk.length = n;
         i->chunk.memblock = pa_memblock_ref(chunk->memblock);
-        
+
         i->channel = channel;
         i->offset = offset;
         i->seek_mode = seek_mode;
 #ifdef HAVE_CREDS
         i->with_creds = 0;
 #endif
-        
+
         pa_queue_push(p->send_queue, i);
 
         idx += n;
         length -= n;
     }
-        
+
     p->mainloop->defer_enable(p->defer_event, 1);
+
+finish:
+
+    pa_mutex_unlock(p->mutex);
 }
 
 static void memimport_release_cb(pa_memimport *i, uint32_t block_id, void *userdata) {
@@ -370,10 +400,12 @@ static void memimport_release_cb(pa_memimport *i, uint32_t block_id, void *userd
     pa_pstream *p = userdata;
 
     assert(p);
-    assert(p->ref >= 1);
-    
+    assert(PA_REFCNT_VALUE(p) > 0);
+
+    pa_mutex_lock(p->mutex);
+
     if (p->dead)
-        return;
+        goto finish;
 
 /*     pa_log("Releasing block %u", block_id); */
 
@@ -386,6 +418,10 @@ static void memimport_release_cb(pa_memimport *i, uint32_t block_id, void *userd
 
     pa_queue_push(p->send_queue, item);
     p->mainloop->defer_enable(p->defer_event, 1);
+
+finish:
+
+    pa_mutex_unlock(p->mutex);
 }
 
 static void memexport_revoke_cb(pa_memexport *e, uint32_t block_id, void *userdata) {
@@ -393,13 +429,15 @@ static void memexport_revoke_cb(pa_memexport *e, uint32_t block_id, void *userda
     pa_pstream *p = userdata;
 
     assert(p);
-    assert(p->ref >= 1);
-    
+    assert(PA_REFCNT_VALUE(p) > 0);
+
+    pa_mutex_lock(p->mutex);
+
     if (p->dead)
-        return;
+        goto finish;
 
 /*     pa_log("Revoking block %u", block_id); */
-    
+
     item = pa_xnew(struct item_info, 1);
     item->type = PA_PSTREAM_ITEM_SHMREVOKE;
     item->block_id = block_id;
@@ -409,14 +447,19 @@ static void memexport_revoke_cb(pa_memexport *e, uint32_t block_id, void *userda
 
     pa_queue_push(p->send_queue, item);
     p->mainloop->defer_enable(p->defer_event, 1);
+
+finish:
+
+    pa_mutex_unlock(p->mutex);
 }
 
 static void prepare_next_write_item(pa_pstream *p) {
     assert(p);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
     if (!(p->write.current = pa_queue_pop(p->send_queue)))
         return;
-    
+
     p->write.index = 0;
     p->write.data = NULL;
 
@@ -425,9 +468,9 @@ static void prepare_next_write_item(pa_pstream *p) {
     p->write.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_HI] = 0;
     p->write.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_LO] = 0;
     p->write.descriptor[PA_PSTREAM_DESCRIPTOR_FLAGS] = 0;
-    
+
     if (p->write.current->type == PA_PSTREAM_ITEM_PACKET) {
-        
+
         assert(p->write.current->packet);
         p->write.data = p->write.current->packet->data;
         p->write.descriptor[PA_PSTREAM_DESCRIPTOR_LENGTH] = htonl(p->write.current->packet->length);
@@ -441,14 +484,14 @@ static void prepare_next_write_item(pa_pstream *p) {
 
         p->write.descriptor[PA_PSTREAM_DESCRIPTOR_FLAGS] = htonl(PA_FLAG_SHMREVOKE);
         p->write.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_HI] = htonl(p->write.current->block_id);
-        
+
     } else {
         uint32_t flags;
         int send_payload = 1;
-        
+
         assert(p->write.current->type == PA_PSTREAM_ITEM_MEMBLOCK);
         assert(p->write.current->chunk.memblock);
-        
+
         p->write.descriptor[PA_PSTREAM_DESCRIPTOR_CHANNEL] = htonl(p->write.current->channel);
         p->write.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_HI] = htonl((uint32_t) (((uint64_t) p->write.current->offset) >> 32));
         p->write.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_LO] = htonl((uint32_t) ((uint64_t) p->write.current->offset));
@@ -467,15 +510,15 @@ static void prepare_next_write_item(pa_pstream *p) {
                                  &shm_id,
                                  &offset,
                                  &length) >= 0) {
-                
+
                 flags |= PA_FLAG_SHMDATA;
                 send_payload = 0;
-                
+
                 p->write.shm_info[PA_PSTREAM_SHM_BLOCKID] = htonl(block_id);
                 p->write.shm_info[PA_PSTREAM_SHM_SHMID] = htonl(shm_id);
                 p->write.shm_info[PA_PSTREAM_SHM_INDEX] = htonl((uint32_t) (offset + p->write.current->chunk.index));
                 p->write.shm_info[PA_PSTREAM_SHM_LENGTH] = htonl((uint32_t) p->write.current->chunk.length);
-                
+
                 p->write.descriptor[PA_PSTREAM_DESCRIPTOR_LENGTH] = htonl(sizeof(p->write.shm_info));
                 p->write.data = p->write.shm_info;
             }
@@ -487,7 +530,7 @@ static void prepare_next_write_item(pa_pstream *p) {
             p->write.descriptor[PA_PSTREAM_DESCRIPTOR_LENGTH] = htonl(p->write.current->chunk.length);
             p->write.data = (uint8_t*) p->write.current->chunk.memblock->data + p->write.current->chunk.index;
         }
-        
+
         p->write.descriptor[PA_PSTREAM_DESCRIPTOR_FLAGS] = htonl(flags);
     }
 
@@ -501,7 +544,9 @@ static int do_write(pa_pstream *p) {
     void *d;
     size_t l;
     ssize_t r;
+
     assert(p);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
     if (!p->write.current)
         prepare_next_write_item(p);
@@ -514,13 +559,13 @@ static int do_write(pa_pstream *p) {
         l = PA_PSTREAM_DESCRIPTOR_SIZE - p->write.index;
     } else {
         assert(p->write.data);
-    
+
         d = (uint8_t*) p->write.data + p->write.index - PA_PSTREAM_DESCRIPTOR_SIZE;
         l = ntohl(p->write.descriptor[PA_PSTREAM_DESCRIPTOR_LENGTH]) - (p->write.index - PA_PSTREAM_DESCRIPTOR_SIZE);
     }
 
     assert(l > 0);
-        
+
 #ifdef HAVE_CREDS
     if (p->send_creds_now) {
 
@@ -550,9 +595,11 @@ static int do_write(pa_pstream *p) {
 
 static int do_read(pa_pstream *p) {
     void *d;
-    size_t l; 
+    size_t l;
     ssize_t r;
+
     assert(p);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
     if (p->read.index < PA_PSTREAM_DESCRIPTOR_SIZE) {
         d = (uint8_t*) p->read.descriptor + p->read.index;
@@ -566,7 +613,7 @@ static int do_read(pa_pstream *p) {
 #ifdef HAVE_CREDS
     {
         int b = 0;
-        
+
         if ((r = pa_iochannel_read_with_creds(p->io, d, l, &p->read_creds, &b)) <= 0)
             return -1;
 
@@ -576,7 +623,7 @@ static int do_read(pa_pstream *p) {
     if ((r = pa_iochannel_read(p->io, d, l)) <= 0)
         return -1;
 #endif
-    
+
     p->read.index += r;
 
     if (p->read.index == PA_PSTREAM_DESCRIPTOR_SIZE) {
@@ -585,22 +632,22 @@ static int do_read(pa_pstream *p) {
 
         flags = ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_FLAGS]);
 
-        if (!p->import && (flags & PA_FLAG_SHMMASK) != 0) {
+        if (!p->use_shm && (flags & PA_FLAG_SHMMASK) != 0) {
             pa_log_warn("Recieved SHM frame on a socket where SHM is disabled.");
             return -1;
         }
-        
+
         if (flags == PA_FLAG_SHMRELEASE) {
 
             /* This is a SHM memblock release frame with no payload */
 
 /*             pa_log("Got release frame for %u", ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_HI])); */
-            
+
             assert(p->export);
             pa_memexport_process_release(p->export, ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_HI]));
 
             goto frame_done;
-            
+
         } else if (flags == PA_FLAG_SHMREVOKE) {
 
             /* This is a SHM memblock revoke frame with no payload */
@@ -614,68 +661,68 @@ static int do_read(pa_pstream *p) {
         }
 
         length = ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_LENGTH]);
-        
-        if (length > FRAME_SIZE_MAX_ALLOW) {
-            pa_log_warn("Recieved invalid frame size : %lu", (unsigned long) length);
+
+        if (length > FRAME_SIZE_MAX_ALLOW || length <= 0) {
+            pa_log_warn("Recieved invalid frame size: %lu", (unsigned long) length);
             return -1;
         }
-        
+
         assert(!p->read.packet && !p->read.memblock);
 
         channel = ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_CHANNEL]);
-        
+
         if (channel == (uint32_t) -1) {
 
             if (flags != 0) {
                 pa_log_warn("Received packet frame with invalid flags value.");
                 return -1;
             }
-            
+
             /* Frame is a packet frame */
             p->read.packet = pa_packet_new(length);
             p->read.data = p->read.packet->data;
-            
+
         } else {
 
             if ((flags & PA_FLAG_SEEKMASK) > PA_SEEK_RELATIVE_END) {
                 pa_log_warn("Received memblock frame with invalid seek mode.");
                 return -1;
             }
-            
+
             if ((flags & PA_FLAG_SHMMASK) == PA_FLAG_SHMDATA) {
 
                 if (length != sizeof(p->read.shm_info)) {
                     pa_log_warn("Recieved SHM memblock frame with Invalid frame length.");
                     return -1;
                 }
-            
+
                 /* Frame is a memblock frame referencing an SHM memblock */
                 p->read.data = p->read.shm_info;
 
             } else if ((flags & PA_FLAG_SHMMASK) == 0) {
 
                 /* Frame is a memblock frame */
-                
+
                 p->read.memblock = pa_memblock_new(p->mempool, length);
                 p->read.data = p->read.memblock->data;
             } else {
-                
+
                 pa_log_warn("Recieved memblock frame with invalid flags value.");
                 return -1;
             }
         }
-            
+
     } else if (p->read.index > PA_PSTREAM_DESCRIPTOR_SIZE) {
         /* Frame payload available */
-        
+
         if (p->read.memblock && p->recieve_memblock_callback) {
 
             /* Is this memblock data? Than pass it to the user */
             l = (p->read.index - r) < PA_PSTREAM_DESCRIPTOR_SIZE ? p->read.index - PA_PSTREAM_DESCRIPTOR_SIZE : (size_t) r;
-                
+
             if (l > 0) {
                 pa_memchunk chunk;
-                
+
                 chunk.memblock = p->read.memblock;
                 chunk.index = p->read.index - PA_PSTREAM_DESCRIPTOR_SIZE - l;
                 chunk.length = l;
@@ -686,7 +733,7 @@ static int do_read(pa_pstream *p) {
                     offset = (int64_t) (
                             (((uint64_t) ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_HI])) << 32) |
                             (((uint64_t) ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_LO]))));
-                    
+
                     p->recieve_memblock_callback(
                         p,
                         ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_CHANNEL]),
@@ -705,14 +752,14 @@ static int do_read(pa_pstream *p) {
 
         /* Frame complete */
         if (p->read.index >= ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_LENGTH]) + PA_PSTREAM_DESCRIPTOR_SIZE) {
-            
+
             if (p->read.memblock) {
 
                 /* This was a memblock frame. We can unref the memblock now */
                 pa_memblock_unref(p->read.memblock);
 
             } else if (p->read.packet) {
-                
+
                 if (p->recieve_packet_callback)
 #ifdef HAVE_CREDS
                     p->recieve_packet_callback(p, p->read.packet, p->read_creds_valid ? &p->read_creds : NULL, p->recieve_packet_callback_userdata);
@@ -723,7 +770,7 @@ static int do_read(pa_pstream *p) {
                 pa_packet_unref(p->read.packet);
             } else {
                 pa_memblock *b;
-                
+
                 assert((ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_FLAGS]) & PA_FLAG_SHMMASK) == PA_FLAG_SHMDATA);
 
                 assert(p->import);
@@ -741,7 +788,7 @@ static int do_read(pa_pstream *p) {
                 if (p->recieve_memblock_callback) {
                     int64_t offset;
                     pa_memchunk chunk;
-                    
+
                     chunk.memblock = b;
                     chunk.index = 0;
                     chunk.length = b->length;
@@ -749,7 +796,7 @@ static int do_read(pa_pstream *p) {
                     offset = (int64_t) (
                             (((uint64_t) ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_HI])) << 32) |
                             (((uint64_t) ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_OFFSET_LO]))));
-                    
+
                     p->recieve_memblock_callback(
                             p,
                             ntohl(p->read.descriptor[PA_PSTREAM_DESCRIPTOR_CHANNEL]),
@@ -782,65 +829,83 @@ frame_done:
 
 void pa_pstream_set_die_callback(pa_pstream *p, pa_pstream_notify_cb_t cb, void *userdata) {
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
+    pa_mutex_lock(p->mutex);
     p->die_callback = cb;
     p->die_callback_userdata = userdata;
+    pa_mutex_unlock(p->mutex);
 }
 
-
 void pa_pstream_set_drain_callback(pa_pstream *p, pa_pstream_notify_cb_t cb, void *userdata) {
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
+    pa_mutex_lock(p->mutex);
     p->drain_callback = cb;
     p->drain_callback_userdata = userdata;
+    pa_mutex_unlock(p->mutex);
 }
 
 void pa_pstream_set_recieve_packet_callback(pa_pstream *p, pa_pstream_packet_cb_t cb, void *userdata) {
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
+    pa_mutex_lock(p->mutex);
     p->recieve_packet_callback = cb;
     p->recieve_packet_callback_userdata = userdata;
+    pa_mutex_unlock(p->mutex);
 }
 
 void pa_pstream_set_recieve_memblock_callback(pa_pstream *p, pa_pstream_memblock_cb_t cb, void *userdata) {
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
+    pa_mutex_lock(p->mutex);
     p->recieve_memblock_callback = cb;
     p->recieve_memblock_callback_userdata = userdata;
+    pa_mutex_unlock(p->mutex);
 }
 
 int pa_pstream_is_pending(pa_pstream *p) {
+    int b;
+
     assert(p);
+    assert(PA_REFCNT_VALUE(p) > 0);
+
+    pa_mutex_lock(p->mutex);
 
     if (p->dead)
-        return 0;
+        b = 0;
+    else
+        b = p->write.current || !pa_queue_is_empty(p->send_queue);
 
-    return p->write.current || !pa_queue_is_empty(p->send_queue);
+    pa_mutex_unlock(p->mutex);
+
+    return b;
 }
 
 void pa_pstream_unref(pa_pstream*p) {
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
 
-    if (--p->ref == 0)
+    if (PA_REFCNT_DEC(p) <= 0)
         pstream_free(p);
 }
 
 pa_pstream* pa_pstream_ref(pa_pstream*p) {
     assert(p);
-    assert(p->ref >= 1);
-    
-    p->ref++;
+    assert(PA_REFCNT_VALUE(p) > 0);
+
+    PA_REFCNT_INC(p);
     return p;
 }
 
 void pa_pstream_close(pa_pstream *p) {
     assert(p);
 
+    pa_mutex_lock(p->mutex);
+
     p->dead = 1;
 
     if (p->import) {
@@ -868,17 +933,19 @@ void pa_pstream_close(pa_pstream *p) {
     p->recieve_packet_callback = NULL;
     p->recieve_memblock_callback = NULL;
 
-
+    pa_mutex_unlock(p->mutex);
 }
 
 void pa_pstream_use_shm(pa_pstream *p, int enable) {
     assert(p);
-    assert(p->ref >= 1);
+    assert(PA_REFCNT_VALUE(p) > 0);
+
+    pa_mutex_lock(p->mutex);
 
     p->use_shm = enable;
 
     if (enable) {
-    
+
         if (!p->export)
             p->export = pa_memexport_new(p->mempool, memexport_revoke_cb, p);
 
@@ -888,6 +955,7 @@ void pa_pstream_use_shm(pa_pstream *p, int enable) {
             pa_memexport_free(p->export);
             p->export = NULL;
         }
-        
     }
+
+    pa_mutex_unlock(p->mutex);
 }
