@@ -1,18 +1,21 @@
-/* $Id: padsp.c 1096 2006-07-16 23:20:27Z lennart $ */
+/* $Id: padsp.c 1444 2007-05-23 15:30:34Z lennart $ */
 
 /***
   This file is part of PulseAudio.
- 
+
+  Copyright 2006 Lennart Poettering
+  Copyright 2006-2007 Pierre Ossman <ossman@cendio.se> for Cendio AB
+
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.
- 
+
   PulseAudio is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
- 
+
   You should have received a copy of the GNU Lesser General Public License
   along with PulseAudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -53,6 +56,11 @@
 #include <pulsecore/llist.h>
 #include <pulsecore/gccmacro.h>
 
+/* On some systems SIOCINQ isn't defined, but FIONREAD is just an alias */
+#if !defined(SIOCINQ) && defined(FIONREAD)
+# define SIOCINQ FIONREAD
+#endif
+
 typedef enum {
     FD_INFO_MIXER,
     FD_INFO_STREAM,
@@ -64,7 +72,7 @@ struct fd_info {
     pthread_mutex_t mutex;
     int ref;
     int unusable;
-    
+
     fd_info_type_t type;
     int app_fd, thread_fd;
 
@@ -76,6 +84,8 @@ struct fd_info {
     pa_context *context;
     pa_stream *play_stream;
     pa_stream *rec_stream;
+    int play_precork;
+    int rec_precork;
 
     pa_io_event *io_event;
     pa_io_event_flags_t io_flags;
@@ -88,7 +98,9 @@ struct fd_info {
     pa_cvolume sink_volume, source_volume;
     uint32_t sink_index, source_index;
     int volume_modify_count;
-    
+
+    int optr_n_blocks;
+
     PA_LLIST_FIELDS(fd_info);
 };
 
@@ -184,26 +196,26 @@ do { \
 
 #define CONTEXT_CHECK_DEAD_GOTO(i, label) do { \
 if (!(i)->context || pa_context_get_state((i)->context) != PA_CONTEXT_READY) { \
-    debug(DEBUG_LEVEL_NORMAL, __FILE__": Not connected: %s", (i)->context ? pa_strerror(pa_context_errno((i)->context)) : "NULL"); \
+    debug(DEBUG_LEVEL_NORMAL, __FILE__": Not connected: %s\n", (i)->context ? pa_strerror(pa_context_errno((i)->context)) : "NULL"); \
     goto label; \
 } \
-} while(0);
+} while(0)
 
 #define PLAYBACK_STREAM_CHECK_DEAD_GOTO(i, label) do { \
 if (!(i)->context || pa_context_get_state((i)->context) != PA_CONTEXT_READY || \
     !(i)->play_stream || pa_stream_get_state((i)->play_stream) != PA_STREAM_READY) { \
-    debug(DEBUG_LEVEL_NORMAL, __FILE__": Not connected: %s", (i)->context ? pa_strerror(pa_context_errno((i)->context)) : "NULL"); \
+    debug(DEBUG_LEVEL_NORMAL, __FILE__": Not connected: %s\n", (i)->context ? pa_strerror(pa_context_errno((i)->context)) : "NULL"); \
     goto label; \
 } \
-} while(0);
+} while(0)
 
 #define RECORD_STREAM_CHECK_DEAD_GOTO(i, label) do { \
 if (!(i)->context || pa_context_get_state((i)->context) != PA_CONTEXT_READY || \
     !(i)->rec_stream || pa_stream_get_state((i)->rec_stream) != PA_STREAM_READY) { \
-    debug(DEBUG_LEVEL_NORMAL, __FILE__": Not connected: %s", (i)->context ? pa_strerror(pa_context_errno((i)->context)) : "NULL"); \
+    debug(DEBUG_LEVEL_NORMAL, __FILE__": Not connected: %s\n", (i)->context ? pa_strerror(pa_context_errno((i)->context)) : "NULL"); \
     goto label; \
 } \
-} while(0);
+} while(0)
 
 static void debug(int level, const char *format, ...) PA_GCC_PRINTF_ATTR(2,3);
 
@@ -241,7 +253,7 @@ static int padsp_disabled(void) {
      * -> disable /dev/dsp emulation, bit 2 -> disable /dev/sndstat
      * emulation, bit 3 -> disable /dev/mixer emulation. Hence a value
      * of 7 disables padsp entirely. */
-    
+
     pthread_mutex_lock(&func_mutex);
     if (!sym_resolved) {
         sym = (int*) dlsym(RTLD_DEFAULT, "__padsp_disabled__");
@@ -252,14 +264,14 @@ static int padsp_disabled(void) {
 
     if (!sym)
         return 0;
-    
+
     return *sym;
 }
 
 static int dsp_cloak_enable(void) {
     if (padsp_disabled() & 1)
         return 0;
-    
+
     if (getenv("PADSP_NO_DSP"))
         return 0;
 
@@ -295,7 +307,7 @@ static int function_enter(void) {
     /* Avoid recursive calls */
     static pthread_once_t recursion_key_once = PTHREAD_ONCE_INIT;
     pthread_once(&recursion_key_once, recursion_key_alloc);
-    
+
     if (pthread_getspecific(recursion_key))
         return 0;
 
@@ -313,10 +325,10 @@ static void fd_info_free(fd_info *i) {
     debug(DEBUG_LEVEL_NORMAL, __FILE__": freeing fd info (fd=%i)\n", i->app_fd);
 
     dsp_drain(i);
-    
+
     if (i->mainloop)
         pa_threaded_mainloop_stop(i->mainloop);
-    
+
     if (i->play_stream) {
         pa_stream_disconnect(i->play_stream);
         pa_stream_unref(i->play_stream);
@@ -331,7 +343,7 @@ static void fd_info_free(fd_info *i) {
         pa_context_disconnect(i->context);
         pa_context_unref(i->context);
     }
-    
+
     if (i->mainloop)
         pa_threaded_mainloop_free(i->mainloop);
 
@@ -353,7 +365,7 @@ static void fd_info_free(fd_info *i) {
 
 static fd_info *fd_info_ref(fd_info *i) {
     assert(i);
-    
+
     pthread_mutex_lock(&i->mutex);
     assert(i->ref >= 1);
     i->ref++;
@@ -397,7 +409,7 @@ static void context_state_cb(pa_context *c, void *userdata) {
 
 static void reset_params(fd_info *i) {
     assert(i);
-    
+
     i->sample_spec.format = PA_SAMPLE_U8;
     i->sample_spec.channels = 1;
     i->sample_spec.rate = 8000;
@@ -411,7 +423,7 @@ static const char *client_name(char *buf, size_t n) {
 
     if ((e = getenv("PADSP_CLIENT_NAME")))
         return e;
-    
+
     if (pa_get_binary_name(p, sizeof(p)))
         snprintf(buf, n, "OSS Emulation[%s]", p);
     else
@@ -433,7 +445,7 @@ static void atfork_prepare(void) {
     fd_info *i;
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": atfork_prepare() enter\n");
-    
+
     function_enter();
 
     pthread_mutex_lock(&fd_infos_mutex);
@@ -445,13 +457,13 @@ static void atfork_prepare(void) {
 
     pthread_mutex_lock(&func_mutex);
 
-    
+
     debug(DEBUG_LEVEL_NORMAL, __FILE__": atfork_prepare() exit\n");
 }
 
 static void atfork_parent(void) {
     fd_info *i;
-    
+
     debug(DEBUG_LEVEL_NORMAL, __FILE__": atfork_parent() enter\n");
 
     pthread_mutex_unlock(&func_mutex);
@@ -464,19 +476,19 @@ static void atfork_parent(void) {
     pthread_mutex_unlock(&fd_infos_mutex);
 
     function_exit();
-    
+
     debug(DEBUG_LEVEL_NORMAL, __FILE__": atfork_parent() exit\n");
 }
 
 static void atfork_child(void) {
     fd_info *i;
-    
+
     debug(DEBUG_LEVEL_NORMAL, __FILE__": atfork_child() enter\n");
 
     /* We do only the bare minimum to get all fds closed */
     pthread_mutex_init(&func_mutex, NULL);
     pthread_mutex_init(&fd_infos_mutex, NULL);
-    
+
     for (i = fd_infos; i; i = i->next) {
         pthread_mutex_init(&i->mutex, NULL);
 
@@ -549,7 +561,7 @@ static fd_info* fd_info_new(fd_info_type_t type, int *_errno) {
     signal(SIGPIPE, SIG_IGN); /* Yes, ugly as hell */
 
     pthread_once(&install_atfork_once, install_atfork);
-    
+
     if (!(i = malloc(sizeof(fd_info)))) {
         *_errno = ENOMEM;
         goto fail;
@@ -562,6 +574,8 @@ static fd_info* fd_info_new(fd_info_type_t type, int *_errno) {
     i->context = NULL;
     i->play_stream = NULL;
     i->rec_stream = NULL;
+    i->play_precork = 0;
+    i->rec_precork = 0;
     i->io_event = NULL;
     i->io_flags = 0;
     pthread_mutex_init(&i->mutex, NULL);
@@ -574,6 +588,7 @@ static fd_info* fd_info_new(fd_info_type_t type, int *_errno) {
     i->volume_modify_count = 0;
     i->sink_index = (uint32_t) -1;
     i->source_index = (uint32_t) -1;
+    i->optr_n_blocks = 0;
     PA_LLIST_INIT(fd_info, i);
 
     reset_params(i);
@@ -630,12 +645,12 @@ static fd_info* fd_info_new(fd_info_type_t type, int *_errno) {
 unlock_and_fail:
 
     pa_threaded_mainloop_unlock(i->mainloop);
-    
+
 fail:
 
     if (i)
         fd_info_unref(i);
-    
+
     return NULL;
 }
 
@@ -663,7 +678,7 @@ static fd_info* fd_info_find(int fd) {
     fd_info *i;
 
     pthread_mutex_lock(&fd_infos_mutex);
-    
+
     for (i = fd_infos; i; i = i->next)
         if (i->app_fd == fd && !i->unusable) {
             fd_info_ref(i);
@@ -671,7 +686,7 @@ static fd_info* fd_info_find(int fd) {
         }
 
     pthread_mutex_unlock(&fd_infos_mutex);
-    
+
     return i;
 }
 
@@ -899,9 +914,21 @@ static void stream_state_cb(pa_stream *s, void * userdata) {
         case PA_STREAM_READY:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": stream established.\n");
             break;
-            
+
         case PA_STREAM_FAILED:
-            debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_connect_playback() failed: %s\n", pa_strerror(pa_context_errno(i->context)));
+            if (s == i->play_stream) {
+                debug(DEBUG_LEVEL_NORMAL,
+                    __FILE__": pa_stream_connect_playback() failed: %s\n",
+                    pa_strerror(pa_context_errno(i->context)));
+                pa_stream_unref(i->play_stream);
+                i->play_stream = NULL;
+            } else if (s == i->rec_stream) {
+                debug(DEBUG_LEVEL_NORMAL,
+                    __FILE__": pa_stream_connect_record() failed: %s\n",
+                    pa_strerror(pa_context_errno(i->context)));
+                pa_stream_unref(i->rec_stream);
+                i->rec_stream = NULL;
+            }
             fd_info_shutdown(i);
             break;
 
@@ -914,8 +941,8 @@ static void stream_state_cb(pa_stream *s, void * userdata) {
 
 static int create_playback_stream(fd_info *i) {
     pa_buffer_attr attr;
-    int n;
-    
+    int n, flags;
+
     assert(i);
 
     fix_metrics(i);
@@ -934,8 +961,13 @@ static int create_playback_stream(fd_info *i) {
     attr.tlength = i->fragment_size * i->n_fragments;
     attr.prebuf = i->fragment_size;
     attr.minreq = i->fragment_size;
-    
-    if (pa_stream_connect_playback(i->play_stream, NULL, &attr, PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE, NULL, NULL) < 0) {
+
+    flags = PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE;
+    if (i->play_precork) {
+        flags |= PA_STREAM_START_CORKED;
+        debug(DEBUG_LEVEL_NORMAL, __FILE__": creating stream corked\n");
+    }
+    if (pa_stream_connect_playback(i->play_stream, NULL, &attr, flags, NULL, NULL) < 0) {
         debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_connect_playback() failed: %s\n", pa_strerror(pa_context_errno(i->context)));
         goto fail;
     }
@@ -944,7 +976,7 @@ static int create_playback_stream(fd_info *i) {
     setsockopt(i->app_fd, SOL_SOCKET, SO_SNDBUF, &n, sizeof(n));
     n = i->fragment_size;
     setsockopt(i->thread_fd, SOL_SOCKET, SO_RCVBUF, &n, sizeof(n));
-    
+
     return 0;
 
 fail:
@@ -953,8 +985,8 @@ fail:
 
 static int create_record_stream(fd_info *i) {
     pa_buffer_attr attr;
-    int n;
-    
+    int n, flags;
+
     assert(i);
 
     fix_metrics(i);
@@ -971,9 +1003,14 @@ static int create_record_stream(fd_info *i) {
     memset(&attr, 0, sizeof(attr));
     attr.maxlength = i->fragment_size * (i->n_fragments+1);
     attr.fragsize = i->fragment_size;
-    
-    if (pa_stream_connect_record(i->rec_stream, NULL, &attr, PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE) < 0) {
-        debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_connect_playback() failed: %s\n", pa_strerror(pa_context_errno(i->context)));
+
+    flags = PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE;
+    if (i->rec_precork) {
+        flags |= PA_STREAM_START_CORKED;
+        debug(DEBUG_LEVEL_NORMAL, __FILE__": creating stream corked\n");
+    }
+    if (pa_stream_connect_record(i->rec_stream, NULL, &attr, flags) < 0) {
+        debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_connect_record() failed: %s\n", pa_strerror(pa_context_errno(i->context)));
         goto fail;
     }
 
@@ -981,7 +1018,7 @@ static int create_record_stream(fd_info *i) {
     setsockopt(i->app_fd, SOL_SOCKET, SO_RCVBUF, &n, sizeof(n));
     n = i->fragment_size;
     setsockopt(i->thread_fd, SOL_SOCKET, SO_SNDBUF, &n, sizeof(n));
-    
+
     return 0;
 
 fail:
@@ -995,12 +1032,21 @@ static void free_streams(fd_info *i) {
         pa_stream_disconnect(i->play_stream);
         pa_stream_unref(i->play_stream);
         i->play_stream = NULL;
+        i->io_flags |= PA_IO_EVENT_INPUT;
     }
 
     if (i->rec_stream) {
         pa_stream_disconnect(i->rec_stream);
         pa_stream_unref(i->rec_stream);
         i->rec_stream = NULL;
+        i->io_flags |= PA_IO_EVENT_OUTPUT;
+    }
+
+    if (i->io_event) {
+        pa_mainloop_api *api;
+
+        api = pa_threaded_mainloop_get_api(i->mainloop);
+        api->io_enable(i->io_event, i->io_flags);
     }
 }
 
@@ -1008,7 +1054,7 @@ static void io_event_cb(pa_mainloop_api *api, pa_io_event *e, int fd, pa_io_even
     fd_info *i = userdata;
 
     pa_threaded_mainloop_signal(i->mainloop, 0);
-    
+
     if (flags & PA_IO_EVENT_INPUT) {
 
         if (!i->play_stream) {
@@ -1018,7 +1064,7 @@ static void io_event_cb(pa_mainloop_api *api, pa_io_event *e, int fd, pa_io_even
             if (fd_info_copy_data(i, 0) < 0)
                 goto fail;
         }
-        
+
     } else if (flags & PA_IO_EVENT_OUTPUT) {
 
         if (!i->rec_stream) {
@@ -1033,7 +1079,7 @@ static void io_event_cb(pa_mainloop_api *api, pa_io_event *e, int fd, pa_io_even
         goto fail;
 
     return;
-    
+
 fail:
     /* We can't do anything better than removing the event source */
     fd_info_shutdown(i);
@@ -1083,7 +1129,7 @@ static int dsp_open(int flags, int *_errno) {
 
     if (!(i->io_event = api->io_new(api, i->thread_fd, i->io_flags, io_event_cb, i)))
         goto fail;
-    
+
     pa_threaded_mainloop_unlock(i->mainloop);
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": dsp_open() succeeded, fd=%i\n", i->app_fd);
@@ -1091,7 +1137,7 @@ static int dsp_open(int flags, int *_errno) {
     fd_info_add_to_list(i);
     ret = i->app_fd;
     fd_info_unref(i);
-    
+
     return ret;
 
 fail:
@@ -1099,7 +1145,7 @@ fail:
 
     if (i)
         fd_info_unref(i);
-    
+
     *_errno = EIO;
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": dsp_open() failed\n");
@@ -1121,7 +1167,7 @@ static void sink_info_cb(pa_context *context, const pa_sink_info *si, int eol, v
 
     if (!pa_cvolume_equal(&i->sink_volume, &si->volume))
         i->volume_modify_count++;
-    
+
     i->sink_volume = si->volume;
     i->sink_index = si->index;
 
@@ -1143,7 +1189,7 @@ static void source_info_cb(pa_context *context, const pa_source_info *si, int eo
 
     if (!pa_cvolume_equal(&i->source_volume, &si->volume))
         i->volume_modify_count++;
-    
+
     i->source_volume = si->volume;
     i->source_index = si->index;
 
@@ -1176,13 +1222,13 @@ static int mixer_open(int flags, int *_errno) {
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": mixer_open()\n");
 
-    if (!(i = fd_info_new(FD_INFO_MIXER, _errno))) 
+    if (!(i = fd_info_new(FD_INFO_MIXER, _errno)))
         return -1;
-    
+
     pa_threaded_mainloop_lock(i->mainloop);
 
     pa_context_set_subscribe_callback(i->context, subscribe_cb, i);
-    
+
     if (!(o = pa_context_subscribe(i->context, PA_SUBSCRIPTION_MASK_SINK | PA_SUBSCRIPTION_MASK_SOURCE, context_success_cb, i))) {
         debug(DEBUG_LEVEL_NORMAL, __FILE__": Failed to subscribe to events: %s", pa_strerror(pa_context_errno(i->context)));
         *_errno = EIO;
@@ -1257,7 +1303,7 @@ static int mixer_open(int flags, int *_errno) {
     fd_info_add_to_list(i);
     ret = i->app_fd;
     fd_info_unref(i);
-    
+
     return ret;
 
 fail:
@@ -1268,7 +1314,7 @@ fail:
 
     if (i)
         fd_info_unref(i);
-    
+
     *_errno = EIO;
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": mixer_open() failed\n");
@@ -1306,7 +1352,7 @@ static int sndstat_open(int flags, int *_errno) {
     int e;
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": sndstat_open()\n");
-    
+
     if (flags != O_RDONLY
 #ifdef O_LARGEFILE
 	&& flags != (O_RDONLY|O_LARGEFILE)
@@ -1384,16 +1430,16 @@ int open(const char *filename, int flags, ...) {
     }
 
     function_exit();
-    
+
     if (_errno)
         errno = _errno;
-    
+
     return r;
 }
 
 static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno) {
     int ret = -1;
-    
+
     switch (request) {
         case SOUND_MIXER_READ_DEVMASK :
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SOUND_MIXER_READ_DEVMASK\n");
@@ -1406,7 +1452,7 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
 
             *(int*) argp = SOUND_MASK_IGAIN;
             break;
-            
+
         case SOUND_MIXER_READ_STEREODEVS:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SOUND_MIXER_READ_STEREODEVS\n");
 
@@ -1417,7 +1463,7 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
             if (i->source_volume.channels > 1)
                 *(int*) argp |= SOUND_MASK_IGAIN;
             pa_threaded_mainloop_unlock(i->mainloop);
-            
+
             break;
 
         case SOUND_MIXER_READ_RECSRC:
@@ -1435,7 +1481,7 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
 
             *(int*) argp = 0;
             break;
-    
+
         case SOUND_MIXER_READ_PCM:
         case SOUND_MIXER_READ_IGAIN: {
             pa_cvolume *v;
@@ -1465,14 +1511,14 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
         case SOUND_MIXER_WRITE_IGAIN: {
             pa_cvolume v, *pv;
 
-            if (request == SOUND_MIXER_READ_PCM)
+            if (request == SOUND_MIXER_WRITE_PCM)
                 debug(DEBUG_LEVEL_NORMAL, __FILE__": SOUND_MIXER_WRITE_PCM\n");
             else
                 debug(DEBUG_LEVEL_NORMAL, __FILE__": SOUND_MIXER_WRITE_IGAIN\n");
 
             pa_threaded_mainloop_lock(i->mainloop);
 
-            if (request == SOUND_MIXER_READ_PCM) {
+            if (request == SOUND_MIXER_WRITE_PCM) {
                 v = i->sink_volume;
                 pv = &i->sink_volume;
             } else {
@@ -1486,10 +1532,10 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
             if (!pa_cvolume_equal(pv, &v)) {
                 pa_operation *o;
 
-                if (request == SOUND_MIXER_READ_PCM)
-                    o = pa_context_set_sink_volume_by_index(i->context, i->sink_index, pv, NULL, NULL);
+                if (request == SOUND_MIXER_WRITE_PCM)
+                    o = pa_context_set_sink_volume_by_index(i->context, i->sink_index, pv, context_success_cb, i);
                 else
-                    o = pa_context_set_source_volume_by_index(i->context, i->source_index, pv, NULL, NULL);
+                    o = pa_context_set_source_volume_by_index(i->context, i->source_index, pv, context_success_cb, i);
 
                 if (!o)
                     debug(DEBUG_LEVEL_NORMAL, __FILE__":Failed set volume: %s", pa_strerror(pa_context_errno(i->context)));
@@ -1498,23 +1544,23 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
                     i->operation_success = 0;
                     while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
                         CONTEXT_CHECK_DEAD_GOTO(i, exit_loop);
-                        
+
                         pa_threaded_mainloop_wait(i->mainloop);
                     }
                 exit_loop:
-                    
+
                     if (!i->operation_success)
                         debug(DEBUG_LEVEL_NORMAL, __FILE__": Failed to set volume: %s\n", pa_strerror(pa_context_errno(i->context)));
 
                     pa_operation_unref(o);
                 }
-                
+
                 /* We don't wait for completion here */
                 i->volume_modify_count++;
             }
-            
+
             pa_threaded_mainloop_unlock(i->mainloop);
-            
+
             break;
         }
 
@@ -1531,7 +1577,7 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
             pa_threaded_mainloop_unlock(i->mainloop);
             break;
         }
-            
+
         default:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": unknown ioctl 0x%08lx\n", request);
 
@@ -1540,44 +1586,44 @@ static int mixer_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno
     }
 
     ret = 0;
-    
+
 fail:
-    
+
     return ret;
 }
 
 static int map_format(int *fmt, pa_sample_spec *ss) {
-    
+
     switch (*fmt) {
         case AFMT_MU_LAW:
             ss->format = PA_SAMPLE_ULAW;
             break;
-            
+
         case AFMT_A_LAW:
             ss->format = PA_SAMPLE_ALAW;
             break;
-            
+
         case AFMT_S8:
             *fmt = AFMT_U8;
             /* fall through */
         case AFMT_U8:
             ss->format = PA_SAMPLE_U8;
             break;
-            
+
         case AFMT_U16_BE:
             *fmt = AFMT_S16_BE;
             /* fall through */
         case AFMT_S16_BE:
             ss->format = PA_SAMPLE_S16BE;
             break;
-            
+
         case AFMT_U16_LE:
             *fmt = AFMT_S16_LE;
             /* fall through */
         case AFMT_S16_LE:
             ss->format = PA_SAMPLE_S16LE;
             break;
-            
+
         default:
             ss->format = PA_SAMPLE_S16NE;
             *fmt = AFMT_S16_NE;
@@ -1649,14 +1695,14 @@ static int dsp_flush_socket(fd_info *i) {
 static int dsp_empty_socket(fd_info *i) {
 #ifdef SIOCINQ
     int ret = -1;
-    
+
     /* Empty the socket */
     for (;;) {
         int l;
-        
+
         if (i->thread_fd < 0)
             break;
-        
+
         if (ioctl(i->thread_fd, SIOCINQ, &l) < 0) {
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SIOCINQ: %s\n", strerror(errno));
             break;
@@ -1666,7 +1712,7 @@ static int dsp_empty_socket(fd_info *i) {
             ret = 0;
             break;
         }
-        
+
         pa_threaded_mainloop_wait(i->mainloop);
     }
 
@@ -1683,19 +1729,19 @@ static int dsp_drain(fd_info *i) {
 
     if (!i->mainloop)
         return 0;
-    
+
     debug(DEBUG_LEVEL_NORMAL, __FILE__": Draining.\n");
 
     pa_threaded_mainloop_lock(i->mainloop);
 
     if (dsp_empty_socket(i) < 0)
         goto fail;
-    
+
     if (!i->play_stream)
         goto fail;
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": Really draining.\n");
-        
+
     if (!(o = pa_stream_drain(i->play_stream, stream_success_cb, i))) {
         debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_drain(): %s\n", pa_strerror(pa_context_errno(i->context)));
         goto fail;
@@ -1704,7 +1750,7 @@ static int dsp_drain(fd_info *i) {
     i->operation_success = 0;
     while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
         PLAYBACK_STREAM_CHECK_DEAD_GOTO(i, fail);
-            
+
         pa_threaded_mainloop_wait(i->mainloop);
     }
 
@@ -1714,9 +1760,9 @@ static int dsp_drain(fd_info *i) {
     }
 
     r = 0;
-    
+
 fail:
-    
+
     if (o)
         pa_operation_unref(o);
 
@@ -1738,7 +1784,7 @@ static int dsp_trigger(fd_info *i) {
         goto fail;
 
     debug(DEBUG_LEVEL_NORMAL, __FILE__": Triggering.\n");
-        
+
     if (!(o = pa_stream_trigger(i->play_stream, stream_success_cb, i))) {
         debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_trigger(): %s\n", pa_strerror(pa_context_errno(i->context)));
         goto fail;
@@ -1747,7 +1793,7 @@ static int dsp_trigger(fd_info *i) {
     i->operation_success = 0;
     while (!pa_operation_get_state(o) != PA_OPERATION_DONE) {
         PLAYBACK_STREAM_CHECK_DEAD_GOTO(i, fail);
-            
+
         pa_threaded_mainloop_wait(i->mainloop);
     }
 
@@ -1757,9 +1803,47 @@ static int dsp_trigger(fd_info *i) {
     }
 
     r = 0;
-    
+
 fail:
-    
+
+    if (o)
+        pa_operation_unref(o);
+
+    pa_threaded_mainloop_unlock(i->mainloop);
+
+    return 0;
+}
+
+static int dsp_cork(fd_info *i, pa_stream *s, int b) {
+    pa_operation *o = NULL;
+    int r = -1;
+
+    pa_threaded_mainloop_lock(i->mainloop);
+
+    if (!(o = pa_stream_cork(s, b, stream_success_cb, i))) {
+        debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_cork(): %s\n", pa_strerror(pa_context_errno(i->context)));
+        goto fail;
+    }
+
+    i->operation_success = 0;
+    while (!pa_operation_get_state(o) != PA_OPERATION_DONE) {
+        if (s == i->play_stream)
+            PLAYBACK_STREAM_CHECK_DEAD_GOTO(i, fail);
+        else if (s == i->rec_stream)
+            RECORD_STREAM_CHECK_DEAD_GOTO(i, fail);
+
+        pa_threaded_mainloop_wait(i->mainloop);
+    }
+
+    if (!i->operation_success) {
+        debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_cork(): %s\n", pa_strerror(pa_context_errno(i->context)));
+        goto fail;
+    }
+
+    r = 0;
+
+fail:
+
     if (o)
         pa_operation_unref(o);
 
@@ -1770,11 +1854,21 @@ fail:
 
 static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno) {
     int ret = -1;
-    
+
+    if (i->thread_fd == -1) {
+        /*
+         * We've encountered some fatal error and are just waiting
+         * for a close.
+         */
+        debug(DEBUG_LEVEL_NORMAL, __FILE__": got ioctl 0x%08lx in fatal error state\n", request);
+        *_errno = EIO;
+        return -1;
+    }
+
     switch (request) {
         case SNDCTL_DSP_SETFMT: {
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_SETFMT: %i\n", *(int*) argp);
-            
+
             pa_threaded_mainloop_lock(i->mainloop);
 
             if (*(int*) argp == AFMT_QUERY)
@@ -1787,12 +1881,12 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
             pa_threaded_mainloop_unlock(i->mainloop);
             break;
         }
-            
+
         case SNDCTL_DSP_SPEED: {
             pa_sample_spec ss;
             int valid;
             char t[256];
-            
+
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_SPEED: %i\n", *(int*) argp);
 
             pa_threaded_mainloop_lock(i->mainloop);
@@ -1804,7 +1898,7 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
                 i->sample_spec = ss;
                 free_streams(i);
             }
-            
+
             debug(DEBUG_LEVEL_NORMAL, __FILE__": ss: %s\n", pa_sample_spec_snprint(t, sizeof(t), &i->sample_spec));
 
             pa_threaded_mainloop_unlock(i->mainloop);
@@ -1816,24 +1910,24 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
 
             break;
         }
-            
+
         case SNDCTL_DSP_STEREO:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_STEREO: %i\n", *(int*) argp);
-            
+
             pa_threaded_mainloop_lock(i->mainloop);
-            
+
             i->sample_spec.channels = *(int*) argp ? 2 : 1;
             free_streams(i);
-            
+
             pa_threaded_mainloop_unlock(i->mainloop);
             return 0;
 
         case SNDCTL_DSP_CHANNELS: {
             pa_sample_spec ss;
             int valid;
-            
+
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_CHANNELS: %i\n", *(int*) argp);
-            
+
             pa_threaded_mainloop_lock(i->mainloop);
 
             ss = i->sample_spec;
@@ -1843,7 +1937,7 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
                 i->sample_spec = ss;
                 free_streams(i);
             }
-            
+
             pa_threaded_mainloop_unlock(i->mainloop);
 
             if (!valid) {
@@ -1861,17 +1955,17 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
 
             fix_metrics(i);
             *(int*) argp = i->fragment_size;
-            
+
             pa_threaded_mainloop_unlock(i->mainloop);
-            
+
             break;
 
         case SNDCTL_DSP_SETFRAGMENT:
-            debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_SETFRAGMENT: 0x%8x\n", *(int*) argp);
-            
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_SETFRAGMENT: 0x%08x\n", *(int*) argp);
+
             pa_threaded_mainloop_lock(i->mainloop);
-            
-            i->fragment_size = 1 << (*(int*) argp);
+
+            i->fragment_size = 1 << ((*(int*) argp) & 31);
             i->n_fragments = (*(int*) argp) >> 16;
 
             /* 0x7FFF means that we can set whatever we like */
@@ -1879,15 +1973,15 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
                 i->n_fragments = 12;
 
             free_streams(i);
-            
+
             pa_threaded_mainloop_unlock(i->mainloop);
-            
+
             break;
-            
+
         case SNDCTL_DSP_GETCAPS:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_CAPS\n");
-            
-            *(int*)  argp = DSP_CAP_DUPLEX
+
+            *(int*)  argp = DSP_CAP_DUPLEX | DSP_CAP_TRIGGER
 #ifdef DSP_CAP_MULTI
 	      | DSP_CAP_MULTI
 #endif
@@ -1896,13 +1990,13 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
 
         case SNDCTL_DSP_GETODELAY: {
             int l;
-            
+
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_GETODELAY\n");
-            
+
             pa_threaded_mainloop_lock(i->mainloop);
 
             *(int*) argp = 0;
-            
+
             for (;;) {
                 pa_usec_t usec;
 
@@ -1920,10 +2014,10 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
 
                 pa_threaded_mainloop_wait(i->mainloop);
             }
-            
+
         exit_loop:
 
-#ifdef SIOCINQ            
+#ifdef SIOCINQ
             if (ioctl(i->thread_fd, SIOCINQ, &l) < 0)
                 debug(DEBUG_LEVEL_NORMAL, __FILE__": SIOCINQ failed: %s\n", strerror(errno));
             else
@@ -1938,38 +2032,65 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
 
             break;
         }
-            
+
         case SNDCTL_DSP_RESET: {
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_RESET\n");
-            
+
             pa_threaded_mainloop_lock(i->mainloop);
 
             free_streams(i);
             dsp_flush_socket(i);
-            reset_params(i);
-            
+
+            i->optr_n_blocks = 0;
+
             pa_threaded_mainloop_unlock(i->mainloop);
             break;
         }
-            
+
         case SNDCTL_DSP_GETFMTS: {
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_GETFMTS\n");
-            
+
             *(int*) argp = AFMT_MU_LAW|AFMT_A_LAW|AFMT_U8|AFMT_S16_LE|AFMT_S16_BE;
             break;
         }
 
         case SNDCTL_DSP_POST:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_POST\n");
-            
-            if (dsp_trigger(i) < 0) 
+
+            if (dsp_trigger(i) < 0)
                 *_errno = EIO;
             break;
 
-        case SNDCTL_DSP_SYNC: 
+        case SNDCTL_DSP_SETTRIGGER:
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_SETTRIGGER: 0x%08x\n", *(int*) argp);
+
+            if (!i->io_event) {
+                *_errno = EIO;
+                break;
+            }
+
+            i->play_precork = !((*(int*) argp) & PCM_ENABLE_OUTPUT);
+
+            if (i->play_stream) {
+                if (dsp_cork(i, i->play_stream, !((*(int*) argp) & PCM_ENABLE_OUTPUT)) < 0)
+                    *_errno = EIO;
+                if (dsp_trigger(i) < 0)
+                    *_errno = EIO;
+            }
+
+            i->rec_precork = !((*(int*) argp) & PCM_ENABLE_INPUT);
+
+            if (i->rec_stream) {
+                if (dsp_cork(i, i->rec_stream, !((*(int*) argp) & PCM_ENABLE_INPUT)) < 0)
+                    *_errno = EIO;
+            }
+
+            break;
+
+        case SNDCTL_DSP_SYNC:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_SYNC\n");
-            
-            if (dsp_drain(i) < 0) 
+
+            if (dsp_drain(i) < 0)
                 *_errno = EIO;
 
             break;
@@ -2035,14 +2156,81 @@ static int dsp_ioctl(fd_info *i, unsigned long request, void*argp, int *_errno)
             break;
         }
 
+        case SOUND_PCM_READ_RATE:
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": SOUND_PCM_READ_RATE\n");
+
+            pa_threaded_mainloop_lock(i->mainloop);
+            *(int*) argp = i->sample_spec.rate;
+            pa_threaded_mainloop_unlock(i->mainloop);
+            break;
+
+        case SOUND_PCM_READ_CHANNELS:
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": SOUND_PCM_READ_CHANNELS\n");
+
+            pa_threaded_mainloop_lock(i->mainloop);
+            *(int*) argp = i->sample_spec.channels;
+            pa_threaded_mainloop_unlock(i->mainloop);
+            break;
+
+        case SOUND_PCM_READ_BITS:
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": SOUND_PCM_READ_BITS\n");
+
+            pa_threaded_mainloop_lock(i->mainloop);
+            *(int*) argp = pa_sample_size(&i->sample_spec)*8;
+            pa_threaded_mainloop_unlock(i->mainloop);
+            break;
+
+        case SNDCTL_DSP_GETOPTR: {
+            count_info *info;
+
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_GETOPTR\n");
+
+            info = (count_info*) argp;
+            memset(info, 0, sizeof(*info));
+
+            pa_threaded_mainloop_lock(i->mainloop);
+
+            for (;;) {
+                pa_usec_t usec;
+
+                PLAYBACK_STREAM_CHECK_DEAD_GOTO(i, exit_loop);
+
+                if (pa_stream_get_time(i->play_stream, &usec) >= 0) {
+                    size_t k = pa_usec_to_bytes(usec, &i->sample_spec);
+                    int m;
+
+                    info->bytes = (int) k;
+                    m = k / i->fragment_size;
+                    info->blocks = m - i->optr_n_blocks;
+                    i->optr_n_blocks = m;
+
+                    break;
+                }
+
+                if (pa_context_errno(i->context) != PA_ERR_NODATA) {
+                    debug(DEBUG_LEVEL_NORMAL, __FILE__": pa_stream_get_latency(): %s\n", pa_strerror(pa_context_errno(i->context)));
+                    break;
+                }
+
+                pa_threaded_mainloop_wait(i->mainloop);
+            }
+
+            pa_threaded_mainloop_unlock(i->mainloop);
+
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": GETOPTR bytes=%i, blocks=%i, ptr=%i\n", info->bytes, info->blocks, info->ptr);
+
+            break;
+        }
+
         case SNDCTL_DSP_GETIPTR:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": invalid ioctl SNDCTL_DSP_GETIPTR\n");
             goto inval;
 
-        case SNDCTL_DSP_GETOPTR:
-            debug(DEBUG_LEVEL_NORMAL, __FILE__": invalid ioctl SNDCTL_DSP_GETOPTR\n");
-            goto inval;
-
+        case SNDCTL_DSP_SETDUPLEX:
+            debug(DEBUG_LEVEL_NORMAL, __FILE__": SNDCTL_DSP_SETDUPLEX\n");
+	    /* this is a no-op */
+	    break;
+	
         default:
             debug(DEBUG_LEVEL_NORMAL, __FILE__": unknown ioctl 0x%08lx\n", request);
 
@@ -2052,9 +2240,9 @@ inval:
     }
 
     ret = 0;
-    
+
 fail:
-    
+
     return ret;
 }
 
@@ -2085,14 +2273,14 @@ int ioctl(int fd, unsigned long request, ...) {
         r = mixer_ioctl(i, request, argp, &_errno);
     else
         r = dsp_ioctl(i, request, argp, &_errno);
-    
+
     fd_info_unref(i);
 
     if (_errno)
         errno = _errno;
 
     function_exit();
-    
+
     return r;
 }
 
@@ -2114,13 +2302,20 @@ int close(int fd) {
 
     fd_info_remove_from_list(i);
     fd_info_unref(i);
-    
+
     function_exit();
 
     return 0;
 }
 
 int access(const char *pathname, int mode) {
+
+    if (!pathname) {
+        /* Firefox needs this. See #27 */
+        errno = EFAULT;
+        return -1;
+    }
+
     debug(DEBUG_LEVEL_VERBOSE, __FILE__": access(%s)\n", pathname);
 
     if (strcmp(pathname, "/dev/dsp") != 0 &&
@@ -2149,7 +2344,7 @@ int open64(const char *filename, int flags, ...) {
     mode_t mode = 0;
 
     debug(DEBUG_LEVEL_VERBOSE, __FILE__": open64(%s)\n", filename);
-    
+
     va_start(args, flags);
     if (flags & O_CREAT)
         mode = va_arg(args, mode_t);
@@ -2172,7 +2367,7 @@ FILE* fopen(const char *filename, const char *mode) {
     FILE *f = NULL;
     int fd;
     mode_t m;
-    
+
     debug(DEBUG_LEVEL_VERBOSE, __FILE__": fopen(%s)\n", filename);
 
     if (strcmp(filename, "/dev/dsp") != 0 &&
@@ -2206,7 +2401,7 @@ FILE* fopen(const char *filename, const char *mode) {
         close(fd);
         return NULL;
     }
-    
+
     return f;
 }
 
@@ -2250,9 +2445,9 @@ int fclose(FILE *f) {
     /* Dirty trick to avoid that the fd is not freed twice, once by us
      * and once by the real fclose() */
     i->app_fd = -1;
-    
+
     fd_info_unref(i);
-    
+
     function_exit();
 
     LOAD_FCLOSE_FUNC();
